from typing import Optional, Union

import chardet
from chardet.universaldetector import UniversalDetector

ALLOWED_ENCODINGS = (None, "ascii")


def get_encoding(*filenames: str) -> dict[str, chardet.ResultDict]:
    """Return encoding information for *filenames*.

    E.g.:

    {
        '__init__.py': {'encoding': 'ascii', 'confidence': 1.0, 'language': ''},
        '__main__.py': {'encoding': 'ascii', 'confidence': 1.0, 'language': ''},
        'cli.py': {'encoding': 'ascii', 'confidence': 1.0, 'language': ''}
        'utils.py': {'encoding': 'ascii', 'confidence': 1.0, 'language': ''}
    }

    """
    detector = UniversalDetector()
    encodings = {}
    for filename in filenames:
        detector.reset()
        with open(filename, "rb") as fh:
            for line in fh.readlines():
                detector.feed(line)
                if detector.done:
                    break
        detector.close()
        encodings[filename] = detector.result
    return encodings


def get_non_ascii_files(*filenames: str) -> dict[str, Optional[str]]:
    """Return dict of `filename: encoding` for files with not allowed encoding."""
    encoding_info = get_encoding(*filenames)
    return {
        filename: info["encoding"]
        for filename, info in encoding_info.items()
        if info["encoding"] not in ALLOWED_ENCODINGS
    }


def is_ascii(s: Union[bytes, bytearray]) -> bool:
    """Return `True` if *string* is ASCII."""
    return chardet.detect(s)["encoding"] in ALLOWED_ENCODINGS


def get_non_ascii_words(filename: str) -> set[str]:
    """Return a set of all non-ASCII words from *filename*."""
    words = set()
    with open(filename, "rb") as fh:
        for line in fh.readlines():
            for word in line.strip().split():
                if not is_ascii(word):
                    words.add(word.decode())
    return words



from ski_lint.cli import get_args, run
from ski_lint.utils import (
    get_encoding,
    get_non_ascii_files,
    get_non_ascii_words,
    is_ascii,
)

__all__ = ["get_args", "run", "get_encoding", "get_non_ascii_files", "get_non_ascii_words", "is_ascii"]



"""A pre-commit hook, that rejects files containing non-ASCII characters."""

import logging
import sys
from argparse import ArgumentParser, Namespace
from importlib.metadata import version

from .utils import get_non_ascii_files, get_non_ascii_words

logging.basicConfig(format="{asctime} - {levelname} - {message}", level=logging.INFO, style="{", stream=sys.stdout)
log = logging.getLogger(__name__)

description = __doc__.strip()


def get_args() -> Namespace:
    ap = ArgumentParser(description=description)
    ap.add_argument("--version", action="version", version=version("ski_lint"))
    ap.add_argument(
        "--check",
        action="store_true",
        help="return code is `1`, when non-ASCII files are found",
    )
    ap.add_argument("filenames", nargs="+", metavar="FILENAME", help="path to the files to check")
    args = ap.parse_args()
    return args


def run(*filenames: str, check: bool = False) -> int:
    bad_encodings = get_non_ascii_files(*filenames)
    if bad_encodings:
        for filename, encoding in bad_encodings.items():
            words = get_non_ascii_words(filename)
            words_str = ", ".join([f"'{w}'" for w in sorted(words)])
            log.error(f"- {filename} ({encoding}): {words_str}")
    else:
        log.info("- NON-ASCII CHECK: OK")

    if bad_encodings and check:
        return 1
    else:
        return 0


def main() -> None:
    args = get_args()
    sys.exit(run(*args.filenames, check=args.check))
